<?php

namespace Teknisa\ACL\Repositories;

use Doctrine\DBAL\Connection;
use Teknisa\ACL\Util\InstanceProvider;

class PerfAcessParc {

    const QUERY_MAX_NR_PERF_ACESS_PARC = "SELECT MAX(NRPERFACESSPARC) as MAXNRPERFACESSPARC FROM PERFACESSPARC";

    /** @var Connection */
    protected $connection;

    public function __construct(Connection $connection) {
        $this->connection = $connection;
    }

    public function getMaxNrPerfAcessParc () {
        return $this->connection->fetchAll(self::QUERY_MAX_NR_PERF_ACESS_PARC);
    }

    public function deleteAssociatedPerfAcessParc (Array $nrPerfilAcesso, Array $params) {
        $profilesToDelete = implode(',', $nrPerfilAcesso);
        $sqlDeletePerfParc = "DELETE FROM PERFACESSPARC WHERE NRPERFILACESSO IN (".$profilesToDelete.")";
        return $this->connection->executeQuery($sqlDeletePerfParc, $params);
    }

    public function findPerfAcessParcOperador($filters, $order)
    {
        $query = <<<SQL
            SELECT CDOPERADOR, NMOPERADOR, NRPERFACESSPARC, NRORG, NRPARCNEGOCIO,
                           NRPERFILACESSO, IDATIVO, TO_CHAR(DTVALIDADE, 'DD/MM/YYYY') AS DTVALIDADE
              FROM (SELECT O.CDOPERADOR, O.NMOPERADOR, P.NRPERFACESSPARC, P.NRORG, P.NRPARCNEGOCIO,
                           P.NRPERFILACESSO, P.IDATIVO, P.DTVALIDADE
                      FROM PERFACESSPARC P
                      JOIN OPERADOR O ON (P.NRPARCNEGOCIO = O.NRPARCNEGOCIO AND P.NRORG = O.NRORG)
                     )
              WHERE 1 = 1
              $filters
              $order
SQL;
    
        return InstanceProvider::getEntityManager()->getConnection()->fetchAll($query);
    }
    
    public function findPerfAcessFilial($filters, $order)
    {
        $query = <<<SQL
            SELECT DISTINCT CDOPERADOR, NMOPERADOR, NRORG,
                           NRPERFILACESSO, IDATIVO
              FROM (SELECT O.CDOPERADOR, O.NMOPERADOR, P.NRORG,
                           P.NRPERFILACESSO, P.IDATIVO, P.DTVALIDADE
                      FROM PERFACESSFILI P
                      JOIN OPERADOR O ON (P.CDOPERADOR = O.CDOPERADOR AND P.NRORG = O.NRORG)
                     )
              WHERE 1 = 1
              $filters
              $order
SQL;
        return InstanceProvider::getEntityManager()->getConnection()->fetchAll($query);
    }
    
    public function saveOperadorPerfAccessFili($row, $cdfilial){


        $query = <<<SQL
            INSERT INTO PERFACESSFILI
                        (CDOPERADOR, CDFILIAL, NRPERFILACESSO, NRORG, NRORGINCLUSAO, CDOPERINCLUSAO, DTINCLUSAO, DTVALIDADE)
                 VALUES (:CDOPERADOR, :CDFILIAL, :NRPERFILACESSO, :NRORG, :NRORGINCLUSAO, :CDOPERINCLUSAO, SYSDATE, :DTVALIDADE)
        
SQL;

   
        $params = array (
            "CDOPERADOR"          => $row['CDOPERADOR'],
            "CDFILIAL"            => $cdfilial,
            "NRPERFILACESSO"      => $row['NRPERFILACESSO'],
            "NRORG"               => $row['NRORG'],
            "NRORGINCLUSAO"       => $row['NRORGINCLUSAO'],
            "CDOPERINCLUSAO"      => $row['CDOPERINCLUSAO'],
            "DTVALIDADE"          => $row['DTVALIDADE']
        );
        
        //self::deleteRegistersFilioper($row, $cdfilial);
        if(self::selectRegistersFilioper($row, $cdfilial)){
            self::insertRegistersFilioper($row, $cdfilial);
        }
        self::deleteOperatorPerfAccessFiliByBranch($row, $cdfilial);
        try {
            $std = InstanceProvider::getEntityManager()->getConnection()->prepare($query);
            $std->execute($params);
        } catch (\Exception $e) {
            throw new \Exception($e->getMessage());
        };
    }
    
    public function deleteOperatorPerfAccessFiliByBranch($row, $cdfilial){
    $queryDelete = <<<SQL
            DELETE FROM PERFACESSFILI 
            WHERE 
            CDOPERADOR = :CDOPERADOR 
            AND CDFILIAL = :CDFILIAL 
            AND NRPERFILACESSO = :NRPERFILACESSO 
            AND NRORG = :NRORG
        
SQL;
        
        $params = array (
            "CDOPERADOR"          => $row['CDOPERADOR'],
            "CDFILIAL"            => $cdfilial,
            "NRPERFILACESSO"      => $row['NRPERFILACESSO'],
            "NRORG"               => $row['NRORG']
        );
        try {
            $std = InstanceProvider::getEntityManager()->getConnection()->prepare($queryDelete);
            
            $std->execute($params);
        } catch (\Exception $e) {
            throw new \Exception($e->getMessage());
        };
    }
    
    public function deleteOperatorPerfAccessFili($row){
    $queryDelete = <<<SQL
            DELETE FROM PERFACESSFILI 
            WHERE 
            CDOPERADOR = :CDOPERADOR
            AND NRPERFILACESSO = :NRPERFILACESSO 
            AND NRORG = :NRORG
        
SQL;
        
        $params = array (
            "CDOPERADOR"          => $row['CDOPERADOR'],
            "NRPERFILACESSO"      => $row['NRPERFILACESSO'],
            "NRORG"               => $row['NRORG']
        );
        
        try {
            $std = InstanceProvider::getEntityManager()->getConnection()->prepare($queryDelete);
            
            $std->execute($params);
        } catch (\Exception $e) {
            throw new \Exception($e->getMessage());
        };
    }
    
    public function deleteRegistersFilioper ($row, $cdfilial){
                $query = <<<SQL
           
        
SQL;

        $queryDelete = <<<SQL
            DELETE FROM FILIOPER 
            WHERE 
            CDOPERADOR = :CDOPERADOR 
            AND CDFILIAL = :CDFILIAL
            AND NRORG = :NRORG
        
SQL;

        $params = array (
            "CDOPERADOR"          => $row['CDOPERADOR'],
            "CDFILIAL"            => $cdfilial,
            "NRORG"               => $row['NRORG']
        );
        
        try {
                $std = InstanceProvider::getEntityManager()->getConnection()->prepare($queryDelete);
                $std->execute($params);
                
            } catch (\Exception $e) {
                throw new \Exception($e->getMessage());
            };
    }
        
        
    public function insertRegistersFilioper ($row, $cdfilial){
                $query = <<<SQL
            INSERT INTO FILIOPER
                        (CDOPERADOR, CDFILIAL, NRORG, NRORGINCLUSAO, CDOPERINCLUSAO)
                 VALUES (:CDOPERADOR, :CDFILIAL, :NRORG, :NRORG, :CDOPERINCLUSAO)
        
SQL;

        $params = array (
            "CDOPERADOR"          => $row['CDOPERADOR'],
            "CDFILIAL"            => $cdfilial,
            "NRORG"               => $row['NRORG'],
            "CDOPERINCLUSAO"      => $row['CDOPERINCLUSAO']
        );
        
        try {
                $std = InstanceProvider::getEntityManager()->getConnection()->prepare($query);
                $std->execute($params);
            } catch (\Exception $e) {
                throw new \Exception($e->getMessage());
            };
        }    
    
    public function selectRegistersFilioper($row, $cdfilial)
    {
        $query = <<<SQL
            SELECT COUNT(*) as TOT FROM FILIOPER 
            WHERE 
            CDOPERADOR = :CDOPERADOR 
            AND CDFILIAL = :CDFILIAL
            AND NRORG = :NRORG
SQL;

        $params = array (
            "CDOPERADOR"          => $row['CDOPERADOR'],
            "CDFILIAL"            => $cdfilial,
            "NRORG"               => $row['NRORG']
        );
        
        try {
            $std = InstanceProvider::getEntityManager()->getConnection()->prepare($query);
            $std->execute($params);
            $result = $std->fetch();
            
            if($result['TOT'] > 0){
                return false;
            }else{
                return true;
            }
            
        } catch (\Exception $e) {
            throw new \Exception($e->getMessage());
        };
    }

}